class Storage(list):
    def __init__(self, mongo_client, mongo_db_name, mongo_coll_name):
        super().__init__()

        self.mongo_client = mongo_client
        self.mongo_db_name = mongo_db_name
        self.mongo_coll_name = mongo_coll_name

    def append_row(self, row):
        self.append(row)

    def export_mongo(self, log):
        log.info('- generating {} collection...'.format(self.mongo_coll_name))

        db = getattr(self.mongo_client, self.mongo_db_name)

        coll = getattr(db, self.mongo_coll_name)
        if coll.count():
            coll.drop()

        for row in self:
            coll.insert_one(row)

        return self.mongo_coll_name



from urllib.parse import urlparse, urljoin

import lxml.html

from .exceptions import DarkKeeperParseContentError


def create_content(html):
    try:
        content = lxml.html.fromstring(html)
    except Exception as e:
        raise DarkKeeperParseContentError(e)

    return content


def parse_urls(content, css_selector, base_url):
    start_url = _calculate_start_url(base_url)

    urls = []
    for link in content.cssselect(css_selector):
        url = link.get('href')
        if not url:
            continue

        url = _normalize_url(url, start_url)
        if url not in urls:
            urls.append(url)

    return urls


def parse_text(content, css_selector):
    title = content.cssselect(css_selector)
    if not len(title):
        return

    return title[0].text_content().strip()


def parse_attr(content, css_selector, css_attr):
    mp3_link = content.cssselect(css_selector)
    if not len(mp3_link):
        return

    return mp3_link[0].get(css_attr).strip()


def _calculate_start_url(base_url):
    base_url = urlparse(base_url)

    return '{scheme}://{netloc}'.format(
        scheme=base_url.scheme, netloc=base_url.netloc
    )


def _normalize_url(url, start_url):
    url_obj = urlparse(url)
    if not url_obj.netloc:
        url = urljoin(start_url, url_obj.path)

    return url



import os
import re
from urllib.parse import urlparse

from .exceptions import DarkKeeperCacheReadError


class Cache(object):
    cache_dir = None

    def read(self, url):
        cache_path = self._get_cache_path(url)
        if os.path.isfile(cache_path):
            with open(cache_path, 'rb') as f:
                try:
                    html = f.read()
                except IOError as e:
                    raise DarkKeeperCacheReadError(e)

                return html

    def write(self, url, html):
        cache_path = self._get_cache_path(url)
        with open(cache_path, 'wb') as f:
            f.write(html)

        return cache_path

    def _get_cache_path(self, url):
        cache_dir = self._get_cache_dir(url)

        cache_file = '{}.html'.format(
            re.sub(r'[:|/|?]', '_', url)
        )

        return os.path.join(
            cache_dir, cache_file
        )

    def _get_cache_dir(self, url):
        if self.cache_dir:
            return self.cache_dir

        domain = urlparse(url).netloc
        self.cache_dir = os.path.join(
            os.getcwd(), 'cache', domain
        )
        if not os.path.isdir(self.cache_dir):
            os.makedirs(self.cache_dir, exist_ok=True)

        return self.cache_dir



import datetime
from collections import OrderedDict


class Logger(object):
    def __init__(self, mongo_client, mongo_db_name, mongo_coll_name):
        self.mongo_client = mongo_client
        self.mongo_db_name = mongo_db_name
        self.mongo_coll_name = '{}_log'.format(mongo_coll_name)

        self.coll = self._get_mongo_collection()

    def info(self, msg):
        created = datetime.datetime.now()

        print('{} {}'.format(created, msg))

        self.export_mongo('info', msg, created)

    def export_mongo(self, level, msg, created):
        self.coll.insert_one(OrderedDict([
            ('level', level),
            ('message', msg),
            ('created', created)
        ]))

    def _get_mongo_collection(self):
        db = getattr(self.mongo_client, self.mongo_db_name)

        coll = getattr(db, self.mongo_coll_name)
        if coll.count():
            coll.drop()

        return getattr(db, self.mongo_coll_name)



class Menu(list):
    def __init__(self, base_url, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.append(base_url)

    def append_new_urls(self, urls):
        for url in urls:
            if url not in self:
                self.append(url)



from setuptools import setup

setup(
    name='dark-keeper',
    version='0.2.6',
    packages=['dark_keeper'],
    url='https://github.com/itcrab/dark-keeper',
    license='MIT License',
    author='Arcady Usov',
    author_email='itcrab@gmail.com',
    maintainer='Arcady Usov',
    maintainer_email='itcrab@gmail.com',
    description='Dark Keeper is open source simple web-parser for podcast-sites',
    long_description=open('README.rst').read(),
    install_requires=[
        'cssselect',
        'lxml',
        'pymongo',
        'requests',
    ],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



import random
import time

import requests

from dark_keeper.cache import Cache
from .exceptions import DarkKeeperRequestResponseError


class Request(object):
    def __init__(self, delay, user_agent=None):
        self.delay = delay if isinstance(delay, list) and len(delay) == 2 else [1, 2]

        self.headers = None
        if user_agent:
            self.headers = {
                'User-Agent': user_agent
            }

        self.cache = Cache()

    def receive_html(self, url):
        html = self.cache.read(url)
        if not html:
            self._delay()

            html = self._from_url(url)

            self.cache.write(url, html)

        return html

    def _delay(self):
        time.sleep(random.uniform(self.delay[0], self.delay[1]))

    def _from_url(self, url):
        try:
            response = requests.get(url, headers=self.headers)
            response.raise_for_status()
        except requests.exceptions.RequestException as e:
            raise DarkKeeperRequestResponseError(e)

        html = response.content

        return html



class DarkKeeperError(Exception):
    pass


class DarkKeeperCacheError(DarkKeeperError):
    pass


class DarkKeeperCacheReadError(DarkKeeperCacheError):
    pass


class DarkKeeperParseError(DarkKeeperError):
    pass


class DarkKeeperParseContentError(DarkKeeperParseError):
    pass


class DarkKeeperRequestError(DarkKeeperError):
    pass


class DarkKeeperRequestResponseError(DarkKeeperRequestError):
    pass



from .log import Logger
from .menu import Menu
from .parse import create_content
from .request import Request
from .storage import Storage


class DarkKeeper(object):
    """
    Dark Keeper is simple web-parser for podcast-sites.
    """
    base_url = None

    mongo_client = None
    mongo_db_name = None
    mongo_coll_name = None

    def __init__(self):
        self.menu = Menu(self.base_url)
        self.storage = Storage(
            self.mongo_client,
            self.mongo_db_name,
            self.mongo_coll_name
        )
        self.request = Request(
            [1, 2],  # delay
            'Mozilla/5.0 (Windows NT 10.0; WOW64) '  # user-agent
            'AppleWebKit/537.36 (KHTML, like Gecko) '
            'Chrome/53.0.2785.116 Safari/537.36 OPR/40.0.2308.81'
        )
        self.log = Logger(
            self.mongo_client,
            self.mongo_db_name,
            self.mongo_coll_name
        )

    def run(self):
        self.log.info('Process is started.')

        for index, url in enumerate(self.menu):
            content = self._get_content(url)

            self.parse_menu(content)
            self.parse_content(content)

            self.log.info('url #{index}: {url}'.format(
                index=index, url=url
            ))

        self.storage.export_mongo(self.log)

        self.log.info(
            'Process is finished - check results in MongoDB!\n'
            'database: {db_name}, collection: {coll_name}'.format(
                db_name=self.storage.mongo_db_name,
                coll_name=self.storage.mongo_coll_name
            )
        )

    def parse_menu(self, content):
        raise NotImplementedError()

    def parse_content(self, content):
        raise NotImplementedError()

    def _get_content(self, url):
        html = self.request.receive_html(url)
        content = create_content(html)

        return content
